#!/usr/bin/perl -w

#use strict;
#use warnings;

open(inFile, "$ARGV[0]") or die "Can't open the input file : $!";
open(outFile, ">$ARGV[1]") or die "Can't open the output file : $!";

$num_main=0;
$num_total=0;
while(<inFile>){
	if($_ =~ />()/){	
		$num_total++;
		$title = $_;
		chomp($title);
		next;
	}
	
	if(uc($_) =~ /([ACUG]+)/) {
		$seq_letter = $1;
		next;
	}
	
	if($_ =~ /(.*?)\s+\(([\-\d\.]+)\)/){
		$symble = $1;	
		#$free_energy = $2;
		@struct_symble = divide_structure($symble);
		@main_struct = get_main_struct(\@struct_symble);
		if(scalar(@main_struct) == 2){
			$num_main++;
=clear
			$left_arm = get_match_all_body($main_struct[0]);
			$right_arm = get_match_all_body($main_struct[1]);
			$left_posi = get_position_in_symble($left_arm, $symble);
			$right_posi = get_position_in_symble($right_arm, $symble);
			$left_raw_letter = get_seq_in_raw($seq_letter, $left_posi, length($left_arm));
			$right_raw_letter = get_seq_in_raw($seq_letter, $right_posi, length($right_arm));
			$main_letter = $left_raw_letter.$right_raw_letter;
			$len_main_letter = length $main_letter;
			$gc_main_letter = get_GC_content($main_letter);
			$bp_main_letter = get_left_bracket_number($left_arm);
			$l_b_ratio = $len_main_letter / $bp_main_letter;
			print outFile "$title	SEQ_LENGTH	$len_main_letter	GC_CONTENT	$gc_main_letter	BASEPAIR	$bp_main_letter	FREE_ENERGY	$free_energy	LEN_BP_RATIO	$l_b_ratio\n";
=cut			
			print outFile "$title\n";
			print outFile "$seq_letter\n";
			print outFile "$symble\n";
		}		
	}
}

print "check $num_total queries, and $num_main have not multiple loops\n";
#######################################
# sub fuctions
sub get_GC_content{
	local($seq) = @_;

	my $lower_seq = lc($seq);
	my $num_gc = ($lower_seq =~ tr/[gc]//);
	return ($num_gc / length($seq));
}

sub get_seq_in_raw{
	local($seq, $posi, $len_arm) = @_;
	return (substr($seq, $posi, $len_arm));
}

sub get_position_in_symble{
	local($str, $seq) = @_;
	return (index($seq, $str));
}

sub get_match_all_body{
	local($seq) = @_;
	local($len_seq, $i, $char, $ext_str, $len_ext_str, $body);
	
	$len_seq = length $seq;
	for($i=0; $i<$len_seq; $i++){
		if(substr($seq, $i, 1) ne "\."){
			$ext_str = substr($seq, $i);
			last;	
		}					
	}
	
	$len_ext_str = length $ext_str;
	$body = $ext_str;
	for($i=($len_ext_str-1); $i>-1; $i--){
		(substr($ext_str, $i, 1) eq "\.") ? chop($body) : last;
	}
	
#	print outFile "$body\n";
	
	return $body;			
}

sub get_left_bracket_number{
	local($seq) = @_;
	return ($seq =~ tr/'('//);
}

sub get_right_bracket_number{
	local($seq) = @_;
	return ($seq =~ tr/')'//);
}

sub get_main_struct{
	local($struct) = @_;
	local(@left_bracket, @right_bracket);
	
	my $len_struct = @$struct;
	for(my $i=0; $i<$len_struct; $i=$i+2){	
		push(@left_bracket, get_left_bracket_number($$struct[$i]));
		push(@right_bracket, get_right_bracket_number($$struct[$i+1]));
	}
	
	my $same=0;
	my $len_left_bracket = @left_bracket;
	for($i=0; $i<$len_left_bracket; $i++){
		$same++ if($left_bracket[$i] == $right_bracket[$i]);	
	}

	return ($same == 1) ? @$struct : ();

}

sub divide_structure{
	local($symble) = @_;
	local($bracket_flag, $len_symble, $i, $char, @posi, $len_posi);
	local(@struct);
	
	push(@posi, 0);
	$bracket_flag = '(';
	$len_symble = length $symble;
	for($i=0; $i<$len_symble; $i++){
		$char = substr($symble, $i, 1);
		if($char ne '.' && $char ne $bracket_flag){
			$bracket_flag = $char;
			push(@posi, $i);
		#	print outFile "$i\n";
		}
	}
	
	$len_posi = @posi;
	for($i=0; $i<($len_posi-1); $i++){	
		push(@struct, substr($symble, $posi[$i], $posi[$i+1] - $posi[$i]));
	}
	
	push(@struct, substr($symble, $posi[$len_posi-1]));
	return @struct;
}

###############################################
close inFile or die "can't close the input file : $!";
close outFile or die "can't close the output file : $!"; 
