#!/bin/bash

#Global Variables
THREESVM_DATA_DIR="3SVM_datasets"
MIRNAS_DATA_DIR="miRNAs8.2h"
NCRNAS_DATA_DIR="Rfam7.0"

function training1 {

	THREESVM_MODEL_DIR=$1
	THREESVM_DATA_DIR=$2
	SAMPLE="hsa163_cds168"

	POS_SET=train_hsa_163
	NEG_SET=train_cds_168
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${POS_SET}.rnafold ${POS_SET}.triple
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${NEG_SET}.rnafold ${NEG_SET}.triple
	perl libsvm_format.pl 1 ${POS_SET}.triple ${POS_SET}.train
	perl libsvm_format.pl -1 ${NEG_SET}.triple ${NEG_SET}.train
	cat ${POS_SET}.train ${NEG_SET}.train > ${SAMPLE}.train
	perl fscores.pl ${POS_SET}.triple < ${SAMPLE}.train > ${SAMPLE}.train.fscores
	sort -nr -k`awk 'END {print NF}' ${SAMPLE}.train.fscores` ${SAMPLE}.train.fscores > ${SAMPLE}.train.fscores.temp
	mv -f ${SAMPLE}.train.fscores.temp ${SAMPLE}.train.fscores
	rm -f ${NEG_SET}.triple ${POS_SET}.triple ${POS_SET}.train ${NEG_SET}.train
	
	cp -f ${THREESVM_DATA_DIR}/trainset_hsa163_cds168_org.model ${SAMPLE}.model
	
	# Training Predictions
	svm-predict ${SAMPLE}.train ${SAMPLE}.model ${SAMPLE}.train.predict > ${SAMPLE}.train.predict.stdout
	sed '/labels/d' ${SAMPLE}.train.predict > ${SAMPLE}.train.predict.temp
	cut -d" " -f1 ${SAMPLE}.train | paste ${SAMPLE}.train.predict.temp - > ${SAMPLE}.train.predict
	awk -v auc="NaN" -v type="${SAMPLE}.1train" -v header=1 -f SVM-metrics.awk ${SAMPLE}.train.predict > ${SAMPLE}.metrics
	rm -f ${SAMPLE}.train.predict.temp

	# Testing Predictions for 30 real and 1000 pseudos
	POS_SET=test_hsa_30
	NEG_SET=test_cds_1000
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${POS_SET}.rnafold ${POS_SET}.triple
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${NEG_SET}.rnafold ${NEG_SET}.triple
	perl libsvm_format.pl 1 ${POS_SET}.triple ${POS_SET}.test
	perl libsvm_format.pl -1 ${NEG_SET}.triple ${NEG_SET}.test
	cat ${POS_SET}.test ${NEG_SET}.test > ${SAMPLE}.test
	rm -f ${POS_SET}.test ${NEG_SET}.test
	
	svm-predict ${SAMPLE}.test ${SAMPLE}.model ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.stdout
	sed '/labels/d' ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.temp
	cut -d" " -f1 ${SAMPLE}.test | paste ${SAMPLE}.test.predict.temp - > ${SAMPLE}.test.predict
	awk -v auc="NaN" -v type="${SAMPLE}.1test1" -v header=0 -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${SAMPLE}.metrics
	rm -f ${SAMPLE}.test.predict.temp ${NEG_SET}.triple ${POS_SET}.triple
	rename ${SAMPLE}.test ${SAMPLE}.1test1 ${SAMPLE}.test* 

	# Testing Predictions for 39 real and 2444 pseudos
	POS_SET=39_hsa_miRNAs_one_stemloop
	NEG_SET=genome_chr19
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${POS_SET}.rnafold ${POS_SET}.triple
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${NEG_SET}.rnafold ${NEG_SET}.triple
	perl libsvm_format.pl 1 ${POS_SET}.triple ${POS_SET}.test
	perl libsvm_format.pl -1 ${NEG_SET}.triple ${NEG_SET}.test
	cat ${POS_SET}.test ${NEG_SET}.test > ${SAMPLE}.test
	rm -f ${POS_SET}.test ${NEG_SET}.test
	
	svm-predict ${SAMPLE}.test ${SAMPLE}.model ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.stdout
	sed '/labels/d' ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.temp
	cut -d" " -f1 ${SAMPLE}.test | paste ${SAMPLE}.test.predict.temp - > ${SAMPLE}.test.predict
	awk -v auc="NaN" -v type="${SAMPLE}.1test2" -v header=0 -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${SAMPLE}.metrics
	rm -f ${SAMPLE}.test.predict.temp ${NEG_SET}.triple ${POS_SET}.triple	
	rename ${SAMPLE}.test ${SAMPLE}.1test2 ${SAMPLE}.test* 
	
	mkdir -p ${THREESVM_MODEL_DIR}
	mv -f ${SAMPLE}.* ${THREESVM_MODEL_DIR}
	
}

function training2 {

	THREESVM_MODEL_DIR=$1
	THREESVM_DATA_DIR=$2
	SAMPLE="hsa163_cds168"

	POS_SET=train_hsa_163
	NEG_SET=train_cds_168
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${POS_SET}.rnafold ${POS_SET}.triple
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${NEG_SET}.rnafold ${NEG_SET}.triple
	perl libsvm_format.pl 1 ${POS_SET}.triple ${POS_SET}.train
	perl libsvm_format.pl -1 ${NEG_SET}.triple ${NEG_SET}.train
	cat ${POS_SET}.train ${NEG_SET}.train > ${SAMPLE}.train
	perl fscores.pl ${POS_SET}.triple < ${SAMPLE}.train > ${SAMPLE}.train.fscores
	sort -nr -k`awk 'END {print NF}' ${SAMPLE}.train.fscores` ${SAMPLE}.train.fscores > ${SAMPLE}.train.fscores.temp
	mv -f ${SAMPLE}.train.fscores.temp ${SAMPLE}.train.fscores
	rm -f ${NEG_SET}.triple ${POS_SET}.triple ${POS_SET}.train ${NEG_SET}.train
	
	# Select C and gamma
	python grid.py -log2c -10,15,1 -log2g -15,10,1 -v 5 -m 400 ${SAMPLE}.train | tail -1 > ${SAMPLE}.grid.best
	mv -f ${SAMPLE}.train.out ${SAMPLE}.grid
	rm -f ${SAMPLE}.train.png 

	# Training using hsa163_cds168
	SVM_TRAIN=`awk '{printf "svm-train -b 1 -c %s -g %s", $1, $2}' ${SAMPLE}.grid.best`
	${SVM_TRAIN} ${SAMPLE}.train ${SAMPLE}.model	
	
	# Training Predictions
	svm-predict -b 1 ${SAMPLE}.train ${SAMPLE}.model ${SAMPLE}.train.predict > ${SAMPLE}.train.predict.stdout
	sed '/labels/d' ${SAMPLE}.train.predict > ${SAMPLE}.train.predict.temp
	cut -d" " -f1 ${SAMPLE}.train | paste ${SAMPLE}.train.predict.temp - > ${SAMPLE}.train.predict
	perl ROC.pl 3 4 -1 < ${SAMPLE}.train.predict > ${SAMPLE}.train.roc
	AUC=`perl AUC.pl 1 2 < ${SAMPLE}.train.roc`
	awk -v auc=${AUC} -v type="${SAMPLE}.2train" -v header=1 -f SVM-metrics.awk ${SAMPLE}.train.predict > ${SAMPLE}.metrics
	rm -f ${SAMPLE}.train.predict.temp

	# Testing Predictions for 30 real and 1000 pseudos
	POS_SET=test_hsa_30
	NEG_SET=test_cds_1000
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${POS_SET}.rnafold ${POS_SET}.triple
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${NEG_SET}.rnafold ${NEG_SET}.triple
	perl libsvm_format.pl 1 ${POS_SET}.triple ${POS_SET}.test
	perl libsvm_format.pl -1 ${NEG_SET}.triple ${NEG_SET}.test
	cat ${POS_SET}.test ${NEG_SET}.test > ${SAMPLE}.test
	rm -f ${POS_SET}.test ${NEG_SET}.test
	
	svm-predict -b 1 ${SAMPLE}.test ${SAMPLE}.model ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.stdout
	sed '/labels/d' ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.temp
	cut -d" " -f1 ${SAMPLE}.test | paste ${SAMPLE}.test.predict.temp - > ${SAMPLE}.test.predict
	perl ROC.pl 3 4 -1 < ${SAMPLE}.test.predict > ${SAMPLE}.test.roc
	AUC=`perl AUC.pl 1 2 < ${SAMPLE}.test.roc`
	awk -v auc=${AUC} -v type="${SAMPLE}.2test1" -v header=0 -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${SAMPLE}.metrics
	rm -f ${SAMPLE}.test.predict.temp ${NEG_SET}.triple ${POS_SET}.triple
	rename ${SAMPLE}.test ${SAMPLE}.2test1 ${SAMPLE}.test* 	

	# Testing Predictions for 39 real and 2444 pseudos
	POS_SET=39_hsa_miRNAs_one_stemloop
	NEG_SET=genome_chr19
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${POS_SET}.rnafold ${POS_SET}.triple
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${NEG_SET}.rnafold ${NEG_SET}.triple
	perl libsvm_format.pl 1 ${POS_SET}.triple ${POS_SET}.test
	perl libsvm_format.pl -1 ${NEG_SET}.triple ${NEG_SET}.test
	cat ${POS_SET}.test ${NEG_SET}.test > ${SAMPLE}.test
	rm -f ${POS_SET}.test ${NEG_SET}.test
	
	svm-predict -b 1 ${SAMPLE}.test ${SAMPLE}.model ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.stdout
	sed '/labels/d' ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.temp
	cut -d" " -f1 ${SAMPLE}.test | paste ${SAMPLE}.test.predict.temp - > ${SAMPLE}.test.predict
	perl ROC.pl 3 4 -1 < ${SAMPLE}.test.predict > ${SAMPLE}.test.roc
	AUC=`perl AUC.pl 1 2 < ${SAMPLE}.test.roc`
	awk -v auc=${AUC} -v type="${SAMPLE}.2test2" -v header=0 -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${SAMPLE}.metrics
	rm -f ${SAMPLE}.test.predict.temp ${NEG_SET}.triple ${POS_SET}.triple	
	rename ${SAMPLE}.test ${SAMPLE}.2test2 ${SAMPLE}.test* 	
	
	mkdir -p ${THREESVM_MODEL_DIR}
	mv -f ${SAMPLE}.* ${THREESVM_MODEL_DIR}
	
}

function training3 {
	# Variables
	POS_SET="Homo_sapiens"
	NEG_SET="pseudoMiRNAs"
	POS_SET_DIR="${MIRNAS_DATA_DIR}/rnafold"
	NEG_SET_DIR="pseudoMiRNAs/rnafold"
	POS_NUM=200
	NEG_NUM=$(($POS_NUM*2))
	SAMPLE=${POS_NUM}${POS_SET}_${NEG_NUM}${NEG_SET}
	RAND_SEED=$1
	TIMESTAMP=$2
	SVM_MODEL=$3
	FILEEXT=$4
	THREESVM_DATA_DIR=$5
	
	# Convert triplet format to libsvm format
	perl 2_get_miRNAs_without_multiple_loops.pl ${POS_SET_DIR}/${POS_SET}.${FILEEXT} ${POS_SET}.noml
	perl 2_get_miRNAs_without_multiple_loops.pl ${NEG_SET_DIR}/${NEG_SET}.${FILEEXT} ${NEG_SET}.noml
	perl 3_step_triplet_coding_for_queries.pl ${POS_SET}.noml ${POS_SET}.${FILEEXT}
	perl 3_step_triplet_coding_for_queries.pl ${NEG_SET}.noml ${NEG_SET}.${FILEEXT}
	perl libsvm_format.pl 1 ${POS_SET}.${FILEEXT} ${POS_SET}.svm
	perl libsvm_format.pl -1 ${NEG_SET}.${FILEEXT} ${NEG_SET}.svm
	rm -f ${POS_SET}.noml ${NEG_SET}.noml 
		
	# Select randomly
	python subset.py -s 1 -r ${RAND_SEED} ${POS_SET}.svm ${POS_NUM} ${POS_SET}.svm1 ${POS_SET}.svm2
	python subset.py -s 1 -r ${RAND_SEED} ${NEG_SET}.svm ${NEG_NUM} ${NEG_SET}.svm1 ${NEG_SET}.svm2
	python subset.py -s 1 -r ${RAND_SEED} ${NEG_SET}.svm2 $(expr `cat ${POS_SET}.svm2 | wc -l | sed 's/ //g'` \* 2) ${NEG_SET}.svm3 ${NEG_SET}.svm
	cat ${POS_SET}.svm1 ${NEG_SET}.svm1 > ${SAMPLE}.train.scale
	cat ${POS_SET}.svm2 ${NEG_SET}.svm3 > ${SAMPLE}.test.scale
	rm -f ${POS_SET}.svm ${POS_SET}.svm1 ${POS_SET}.svm2 ${NEG_SET}.svm1 ${NEG_SET}.svm2 ${NEG_SET}.svm3

	# Create F-scores
	perl fscores.pl ${POS_SET}.${FILEEXT} < ${SAMPLE}.train.scale > ${SAMPLE}.train.scale.fscores
	sort -nr -k`awk 'END {print NF}' ${SAMPLE}.train.scale.fscores` ${SAMPLE}.train.scale.fscores > ${SAMPLE}.train.scale.fscores.temp
	mv -f ${SAMPLE}.train.scale.fscores.temp ${SAMPLE}.train.scale.fscores
	rm -f ${POS_SET}.${FILEEXT} ${NEG_SET}.${FILEEXT}
	
	# Select C and gamma
	python grid.py -log2c -10,15,1 -log2g -15,10,1 -v 5 -m 400 ${SAMPLE}.train.scale | tail -1 > ${SAMPLE}.grid.best
	mv -f ${SAMPLE}.train.scale.out ${SAMPLE}.grid
	rm -f ${SAMPLE}.train.scale.png 

	# Training
	SVM_TRAIN=`awk '{printf "svm-train -b 1 -c %s -g %s", $1, $2}' ${SAMPLE}.grid.best`
	${SVM_TRAIN} ${SAMPLE}.train.scale ${SAMPLE}.model

	# Training Predictions
	svm-predict -b 1 ${SAMPLE}.train.scale ${SAMPLE}.model ${SAMPLE}.train.predict > ${SAMPLE}.train.predict.stdout
	sed '/labels/d' ${SAMPLE}.train.predict > ${SAMPLE}.train.predict.temp
	cut -d" " -f1 ${SAMPLE}.train.scale | paste ${SAMPLE}.train.predict.temp - > ${SAMPLE}.train.predict
	perl ROC.pl 3 4 -1 < ${SAMPLE}.train.predict > ${SAMPLE}.train.roc
	AUC=`perl AUC.pl 1 2 < ${SAMPLE}.train.roc`
	awk -v auc=${AUC} -v type="${SAMPLE}.3train" -v header=1 -f SVM-metrics.awk ${SAMPLE}.train.predict > ${SAMPLE}.metrics
	rm -f ${SAMPLE}.train.predict.temp

	# Testing Predictions for 30 real and 1000 pseudos
	POS_SET=test_hsa_30
	NEG_SET=test_cds_1000
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${POS_SET}.rnafold ${POS_SET}.triple
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${NEG_SET}.rnafold ${NEG_SET}.triple
	perl libsvm_format.pl 1 ${POS_SET}.triple ${POS_SET}.3test1
	perl libsvm_format.pl -1 ${NEG_SET}.triple ${NEG_SET}.3test1
	cat ${POS_SET}.3test1 ${NEG_SET}.3test1 > ${SAMPLE}.3test1
	rm -f ${POS_SET}.3test1 ${NEG_SET}.3test1
	
	svm-predict -b 1 ${SAMPLE}.3test1 ${SAMPLE}.model ${SAMPLE}.3test1.predict > ${SAMPLE}.3test1.predict.stdout
	sed '/labels/d' ${SAMPLE}.3test1.predict > ${SAMPLE}.3test1.predict.temp
	cut -d" " -f1 ${SAMPLE}.3test1 | paste ${SAMPLE}.3test1.predict.temp - > ${SAMPLE}.3test1.predict
	perl ROC.pl 3 4 -1 < ${SAMPLE}.3test1.predict > ${SAMPLE}.3test1.roc
	AUC=`perl AUC.pl 1 2 < ${SAMPLE}.3test1.roc`
	awk -v auc=${AUC} -v type="${SAMPLE}.3test1" -v header=0 -f SVM-metrics.awk ${SAMPLE}.3test1.predict >> ${SAMPLE}.metrics
	rm -f ${SAMPLE}.3test1.predict.temp ${NEG_SET}.triple ${POS_SET}.triple
	#rename ${SAMPLE}.test ${SAMPLE}.3test1 ${SAMPLE}.test*	

	# Testing Predictions for 39 real and 2444 pseudos
	POS_SET=39_hsa_miRNAs_one_stemloop
	NEG_SET=genome_chr19
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${POS_SET}.rnafold ${POS_SET}.triple
	perl 3_step_triplet_coding_for_queries.pl ${THREESVM_DATA_DIR}/${NEG_SET}.rnafold ${NEG_SET}.triple
	perl libsvm_format.pl 1 ${POS_SET}.triple ${POS_SET}.3test2
	perl libsvm_format.pl -1 ${NEG_SET}.triple ${NEG_SET}.3test2
	cat ${POS_SET}.3test2 ${NEG_SET}.3test2 > ${SAMPLE}.3test2
	rm -f ${POS_SET}.3test2 ${NEG_SET}.3test2
	
	svm-predict -b 1 ${SAMPLE}.3test2 ${SAMPLE}.model ${SAMPLE}.3test2.predict > ${SAMPLE}.3test2.predict.stdout
	sed '/labels/d' ${SAMPLE}.3test2.predict > ${SAMPLE}.3test2.predict.temp
	cut -d" " -f1 ${SAMPLE}.3test2 | paste ${SAMPLE}.3test2.predict.temp - > ${SAMPLE}.3test2.predict
	perl ROC.pl 3 4 -1 < ${SAMPLE}.3test2.predict > ${SAMPLE}.3test2.roc
	AUC=`perl AUC.pl 1 2 < ${SAMPLE}.3test2.roc`
	awk -v auc=${AUC} -v type="${SAMPLE}.3test2" -v header=0 -f SVM-metrics.awk ${SAMPLE}.3test2.predict >> ${SAMPLE}.metrics
	rm -f ${SAMPLE}.3test2.predict.temp ${NEG_SET}.triple ${POS_SET}.triple
	#rename ${SAMPLE}.3test2 ${SAMPLE}.3test2 ${SAMPLE}.3test2*
	
	# Testing Predictions for remaining real and 2x pseudos
	svm-predict -b 1 ${SAMPLE}.test.scale ${SAMPLE}.model ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.stdout
	sed '/labels/d' ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.temp
	cut -d" " -f1 ${SAMPLE}.test.scale | paste ${SAMPLE}.test.predict.temp - > ${SAMPLE}.test.predict
	perl ROC.pl 3 4 -1 < ${SAMPLE}.test.predict > ${SAMPLE}.test.roc
	AUC=`perl AUC.pl 1 2 < ${SAMPLE}.test.roc`
	awk -v auc=${AUC} -v type="${SAMPLE}.3test3" -v header=0 -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${SAMPLE}.metrics
	rm -f ${SAMPLE}.test.predict.temp
	
	mkdir -p ${TIMESTAMP}-${RAND_SEED}-${SVM_MODEL}
	mv -f ${SAMPLE}.* *.svm ${TIMESTAMP}-${RAND_SEED}-${SVM_MODEL}

}

function testing12 {

	TIMESTAMP=$1
	MODELDIR=$2
	CLASS=$3
	
	for i in ${MIRNAS_DATA_DIR} ${NCRNAS_DATA_DIR} mRNAs ${THREESVM_DATA_DIR}
	do 
		TRIPLESVM_DIR="${TIMESTAMP}_${i}_3SVM${CLASS}"
		mkdir ${TRIPLESVM_DIR}
		
		if [ "${i}" != "${THREESVM_DATA_DIR}" ]
		then 
			RNAFiles=`ls ${i}/rnafold/*.rnafold`
		else
			RNAFiles=`ls ${i}/???.rnafold`
		fi

		for RNAFile in ${RNAFiles}
		do
			RNAFOLDDIR=`dirname "$RNAFile"`
			BASENAME=`basename "$RNAFile" .rnafold`
			
			perl 1_check_query_content.pl ${RNAFOLDDIR}/${BASENAME}.rnafold ${TRIPLESVM_DIR}/${BASENAME}.acgu
	
			if [ "${i}" != "${THREESVM_DATA_DIR}" ]
			then
				perl 2_get_miRNAs_without_multiple_loops.pl ${TRIPLESVM_DIR}/${BASENAME}.acgu ${TRIPLESVM_DIR}/${BASENAME}.noml
			else 
				cp  ${TRIPLESVM_DIR}/${BASENAME}.acgu  ${TRIPLESVM_DIR}/${BASENAME}.noml
			fi
	
			perl 3_step_triplet_coding_for_queries.pl ${TRIPLESVM_DIR}/${BASENAME}.noml ${TRIPLESVM_DIR}/${BASENAME}.te
			diff -B ${TRIPLESVM_DIR}/${BASENAME}.noml ${RNAFOLDDIR}/${BASENAME}.rnafold | sed 's/[><] //' | grep \> > ${TRIPLESVM_DIR}/${BASENAME}.diff
			
			if [ "${i}" != "ncRNAs" ] 
			then
				perl libsvm_format.pl 1 ${TRIPLESVM_DIR}/${BASENAME}.te ${TRIPLESVM_DIR}/${BASENAME}.svmi
			else 
				perl libsvm_format.pl -1 ${TRIPLESVM_DIR}/${BASENAME}.te ${TRIPLESVM_DIR}/${BASENAME}.svmi
			fi					
			
			SVM_PREDICT_OPTION="-b 1 "
			if [ "${CLASS}" -eq 1 ]
			then
				SVM_PREDICT_OPTION=" "
			fi

			svm-predict ${SVM_PREDICT_OPTION} ${TRIPLESVM_DIR}/${BASENAME}.svmi ${MODELDIR}/*.model ${TRIPLESVM_DIR}/${BASENAME}.svmo > ${TRIPLESVM_DIR}/${BASENAME}.svmp

			sed '/labels/d' ${TRIPLESVM_DIR}/${BASENAME}.svmo > ${TRIPLESVM_DIR}/${BASENAME}.svmo.temp
			cut -d" " -f1 ${TRIPLESVM_DIR}/${BASENAME}.svmi | paste ${TRIPLESVM_DIR}/${BASENAME}.svmo.temp - > ${TRIPLESVM_DIR}/${BASENAME}.svmo
			rm -f ${TRIPLESVM_DIR}/${BASENAME}.svmo.temp
			
		done

		STATSFILE=${TRIPLESVM_DIR}.metrics

		for j in `ls ${TRIPLESVM_DIR}/*.acgu`
		do
			BASENAME=`basename "$j" .acgu`

			echo -e ${BASENAME} "\t" \
					`grep -hc \> ${TRIPLESVM_DIR}/${BASENAME}.acgu` "\t" \
					`grep -hc \> ${TRIPLESVM_DIR}/${BASENAME}.diff` "\t" \
					`grep -hc \> ${TRIPLESVM_DIR}/${BASENAME}.noml` | awk 'BEGIN {OFS="\t"} {print $1, $2, $3, $3/$2*100, $4}' >>  ${STATSFILE}1

			awk -v header=0 -f SVM-metrics.awk ${TRIPLESVM_DIR}/${BASENAME}.svmo >> ${STATSFILE}2
		
		done

		echo -e "RNAs\tACGU\tSkipped\tSkipped(%)\tNoML\tTP\tFN\tP\tFP\tTN\tN\t%SE\t%SP\t%FPr\t%PPV\t%NPV\t%ACC\t%Fm\t%MCC" > ${STATSFILE}
		paste ${STATSFILE}1 ${STATSFILE}2 >> ${STATSFILE}
		rm -f ${STATSFILE}1 ${STATSFILE}2

	done
}

function test3_miRNAs {

	POS_SET_DIR="${MIRNAS_DATA_DIR}"
	NEG_SET="pseudoMiRNAs"
	FIRSTTIME=1
	MODEL_DIR=$1
	RAND_SEED=$2
	SVM_MODEL=$3
	FILEEXT=$4

	mkdir -p ${MODEL_DIR}_${POS_SET_DIR}	

	for i in `ls ${POS_SET_DIR}/rnafold/*.${FILEEXT}`
	do
		POS_SET=`basename "$i" .${FILEEXT}`

                #if [ "${POS_SET}" == "sapiens_hp" ]
                #then
                #        continue
                #fi
	
 		perl 2_get_miRNAs_without_multiple_loops.pl ${POS_SET_DIR}/rnafold/${POS_SET}.${FILEEXT} ${POS_SET}.noml
		perl 3_step_triplet_coding_for_queries.pl ${POS_SET}.noml ${POS_SET}.${FILEEXT}
		perl libsvm_format.pl 1 ${POS_SET}.${FILEEXT} ${POS_SET}.svm
		rm -f ${POS_SET}.noml ${POS_SET}.${FILEEXT}
		
		POS_NUM=`cat ${POS_SET}.svm | wc -l | sed 's/ //g'`
		NEG_NUM=$(($POS_NUM*2))
		SAMPLE="${POS_SET}${POS_NUM}_${NEG_SET}${NEG_NUM}"

		if [ ! -e ${MODEL_DIR}/${NEG_SET}.svm.org ]
		then
			cp -f ${MODEL_DIR}/${NEG_SET}.svm ${MODEL_DIR}/${NEG_SET}.svm.org
			FIRSTTIME=0
		fi
		
		if [ "${FIRSTTIME}" -eq 1 ]
		then
			cp -f ${MODEL_DIR}/${NEG_SET}.svm.org ${MODEL_DIR}/${NEG_SET}.svm
			FIRSTTIME=0
		fi
		
		python subset.py -s 1 -r ${RAND_SEED} ${MODEL_DIR}/${NEG_SET}.svm ${NEG_NUM} ${NEG_SET}.svm1 ${NEG_SET}.svm2
		cat ${POS_SET}.svm ${NEG_SET}.svm1 > ${SAMPLE}.test.scale
		rm -f ${POS_SET}.svm ${NEG_SET}.svm1
		mv -f ${NEG_SET}.svm2 ${MODEL_DIR}/${NEG_SET}.svm
		
		svm-predict -b 1 ${SAMPLE}.test.scale ${MODEL_DIR}/*.model ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.stdout
		sed '/labels/d' ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.temp
		cut -d" " -f1 ${SAMPLE}.test.scale | paste ${SAMPLE}.test.predict.temp - > ${SAMPLE}.test.predict
		rm -f ${SAMPLE}.test.predict.temp
	
		if [ ! -e ${MODEL_DIR}_${POS_SET_DIR}.metrics ]
		then
			awk -v type="${SAMPLE}" -v header=1 -f SVM-metrics.awk ${SAMPLE}.test.predict > ${MODEL_DIR}_${POS_SET_DIR}.metrics
		else
			awk -v type="${SAMPLE}" -v header=0 -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${MODEL_DIR}_${POS_SET_DIR}.metrics
		fi
		
		mv -f ${SAMPLE}.* ${MODEL_DIR}_${POS_SET_DIR}
		
	done
}

function test3_RNAs {

	DATASET_DIR="$1"
	CLASS=$2
	MODEL_DIR=$3
	SVM_MODEL=$4
	FILEEXT=$5
	
	mkdir -p ${MODEL_DIR}_${DATASET_DIR}
	
	for i in `ls ${DATASET_DIR}/rnafold/*.${FILEEXT}`
	do
		DATASET=`basename "$i" .${FILEEXT}`
		
 		perl 2_get_miRNAs_without_multiple_loops.pl ${DATASET_DIR}/rnafold/${DATASET}.${FILEEXT} ${DATASET}.noml
		perl 3_step_triplet_coding_for_queries.pl ${DATASET}.noml ${DATASET}.${FILEEXT}
		perl libsvm_format.pl ${CLASS} ${DATASET}.${FILEEXT} ${DATASET}.svm
		rm -f ${DATASET}.noml ${DATASET}.${FILEEXT}
		
		NUM=`cat ${DATASET}.svm | wc -l | sed 's/ //g'`
		SAMPLE="${DATASET}_${NUM}"
		mv -f ${DATASET}.svm ${SAMPLE}.test.svm
		
		mv -f ${SAMPLE}.test.svm ${SAMPLE}.test.scale

		svm-predict -b 1 ${SAMPLE}.test.scale ${MODEL_DIR}/*.model ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.stdout
		sed '/labels/d' ${SAMPLE}.test.predict > ${SAMPLE}.test.predict.temp
		cut -d" " -f1 ${SAMPLE}.test.scale | paste ${SAMPLE}.test.predict.temp - > ${SAMPLE}.test.predict
		rm -f ${SAMPLE}.test.predict.temp
		
		if [ ! -e ${MODEL_DIR}_${DATASET_DIR}.metrics ]
		then
			awk -v type="${SAMPLE}" -v header=1 -f SVM-metrics.awk ${SAMPLE}.test.predict > ${MODEL_DIR}_${DATASET_DIR}.metrics
		else
			awk -v type="${SAMPLE}" -v header=0 -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${MODEL_DIR}_${DATASET_DIR}.metrics
		fi
		
		mv -f ${SAMPLE}.* ${MODEL_DIR}_${DATASET_DIR}
		
	done
}

# 3SVM Version 1 (original training set and model) and 2 (original training set, but updated model)
TIMESTAMP=`date +%H%M%S-%d%m%Y`

training1 "${TIMESTAMP}-3SVM1" ${THREESVM_DATA_DIR}
training2 "${TIMESTAMP}-3SVM2" ${THREESVM_DATA_DIR}
testing12 ${TIMESTAMP} "${TIMESTAMP}-3SVM1" 1
testing12 ${TIMESTAMP} "${TIMESTAMP}-3SVM2" 2

# 3SVM Version 3 (using 200p and 400n)
TIMESTAMP=`date +%H%M%S-%d%m%Y`
NUM=3
SVM_MODEL="3SVM3"
FILEEXT="rnafold"
MODEL_DIR="${TIMESTAMP}-${NUM}-${SVM_MODEL}"

training3 ${NUM} ${TIMESTAMP} ${SVM_MODEL} ${FILEEXT} ${THREESVM_DATA_DIR}
test3_miRNAs ${MODEL_DIR} ${NUM} ${SVM_MODEL} ${FILEEXT}
test3_RNAs ${NCRNAS_DATA_DIR} "-1" ${MODEL_DIR} ${SVM_MODEL} ${FILEEXT}
test3_RNAs mRNAs "-1" ${MODEL_DIR} ${SVM_MODEL} ${FILEEXT}

./Bayes-workflow.bash ${MODEL_DIR} ${FILEEXT}





