#!/bin/bash

function training {

	# Variables
	POS_SET="Homo_sapiens"
	NEG_SET="pseudoMiRNAs"
	POS_NUM=200
	NEG_NUM=$(($POS_NUM*2))
	SAMPLE=${POS_NUM}${POS_SET}_${NEG_NUM}${NEG_SET}
	MODEL_DIR=$1
	FILEEXT=$2
	NUM_VAR=`cat ${MODEL_DIR}/${SAMPLE}.train.scale.fscores | wc -l | sed 's/ //g'`

	#Convert 
	perl Bayes_format.pl ${NUM_VAR} < ${MODEL_DIR}/${SAMPLE}.train.scale > ${SAMPLE}.train.${FILEEXT}
	perl Bayes_format.pl ${NUM_VAR} < ${MODEL_DIR}/${SAMPLE}.test.scale > ${SAMPLE}.test.${FILEEXT}

	#Determining Attribute Domains
	dom -d ${SAMPLE}.train.${FILEEXT} ${SAMPLE}.train.dom
	
	#Inducing a Bayes Classifier
	bci -L1 -d -sr ${SAMPLE}.train.dom ${SAMPLE}.train.${FILEEXT} ${SAMPLE}.train.nbc
	
	#Executing a Bayes Classifier
	bcx -L1 -d -a -p"p-value" -o"%.5f" ${SAMPLE}.train.nbc ${SAMPLE}.train.${FILEEXT} ${SAMPLE}.train.cls
	bcx -L1 -d -a -p"p-value" -o"%.5f" ${SAMPLE}.train.nbc ${SAMPLE}.test.${FILEEXT} ${SAMPLE}.test.cls
	awk 'BEGIN {OFS="\t"} {if (NR != 1) print $(NF-1), $NF, $(NF-2)}' ${SAMPLE}.train.cls > ${SAMPLE}.train.predict
	awk 'BEGIN {OFS="\t"} {if (NR != 1) print $(NF-1), $NF, $(NF-2)}' ${SAMPLE}.test.cls > ${SAMPLE}.test.predict

	for i in `seq 0 0.01 1.01`
	do
		bcx -L1 -d -a -t"${i}" ${SAMPLE}.train.nbc ${SAMPLE}.train.${FILEEXT} ${SAMPLE}.train.cls 2>/dev/null
		awk -v threshold=${i} -f Bayes-metrics.awk ${SAMPLE}.train.cls >> ${SAMPLE}.train.roc1
		bcx -L1 -d -a -t"${i}" ${SAMPLE}.train.nbc ${SAMPLE}.test.${FILEEXT} ${SAMPLE}.test.cls 2>/dev/null
		awk -v threshold=${i} -f Bayes-metrics.awk ${SAMPLE}.test.cls >> ${SAMPLE}.test.roc1
	done
	
	sort -n -k1 ${SAMPLE}.train.roc1 > ${SAMPLE}.train.roc
	sort -n -k1 ${SAMPLE}.test.roc1 > ${SAMPLE}.test.roc
	rm -f ${SAMPLE}.test.roc1 ${SAMPLE}.train.roc1 
	
	#perl ROC.pl 2 3 1 < ${SAMPLE}.train.predict1 > ${SAMPLE}.train.roc
	AUC=`perl AUC.pl 1 2 < ${SAMPLE}.train.roc`	
	awk -v auc=${AUC} -v type="${SAMPLE}.train" -v header=1 -f SVM-metrics.awk ${SAMPLE}.train.predict > ${SAMPLE}.metrics

	#perl ROC.pl 2 3 1 < ${SAMPLE}.test.predict1 > ${SAMPLE}.test.roc
	AUC=`perl AUC.pl 1 2 < ${SAMPLE}.test.roc`	
	awk -v auc=${AUC} -v type="${SAMPLE}.test" -v header=0  -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${SAMPLE}.metrics
	rm -f ${SAMPLE}.train.cls ${SAMPLE}.test.cls ${SAMPLE}.train.predict1 ${SAMPLE}.test.predict1 
	
	mkdir -p ${MODEL_DIR}_BAYES
	mv -f ${SAMPLE}.* ${MODEL_DIR}_BAYES/

}

function test_RNAs {

	DATASET_DIR=$1
	MODEL_DIR=$2
	FILEEXT=$3
	NUM_VAR=`cat ${MODEL_DIR}/*.train.scale.fscores | wc -l | sed 's/ //g'`
	
	mkdir -p ${MODEL_DIR}_${DATASET_DIR}_BAYES

	for i in `ls ${MODEL_DIR}_${DATASET_DIR}/*.test.scale`
	do
		SAMPLE=`basename "$i" .test.scale`
		
		#convert
		perl Bayes_format.pl ${NUM_VAR} < ${MODEL_DIR}_${DATASET_DIR}/${SAMPLE}.test.scale > ${SAMPLE}.test.${FILEEXT}
		
		#Executing a Bayes Classifier
		bcx -L1 -d -a -p"p-value" -o"%.5f" ${MODEL_DIR}_BAYES/*.train.nbc ${SAMPLE}.test.${FILEEXT} ${SAMPLE}.test.cls
		awk 'BEGIN {OFS="\t"} {if (NR != 1) print $(NF-1), $NF, $(NF-2)}' ${SAMPLE}.test.cls > ${SAMPLE}.test.predict
		rm -f ${SAMPLE}.test.cls 
		
		if [ ! -e ${MODEL_DIR}_${DATASET_DIR}_BAYES.metrics ]
		then
			awk -v type="${SAMPLE}" -v header=1 -f SVM-metrics.awk ${SAMPLE}.test.predict > ${MODEL_DIR}_${DATASET_DIR}_BAYES.metrics
		else
			awk -v type="${SAMPLE}" -v header=0 -f SVM-metrics.awk ${SAMPLE}.test.predict >> ${MODEL_DIR}_${DATASET_DIR}_BAYES.metrics
		fi
		
		mv -f ${SAMPLE}.* ${MODEL_DIR}_${DATASET_DIR}_BAYES
		
	done
}

MODEL_DIR=$1
FILEEXT=$2

training ${MODEL_DIR} ${FILEEXT}

test_RNAs miRNAs8.2h ${MODEL_DIR} ${MODEL_DIR} ${FILEEXT}
test_RNAs Rfam7.0 ${MODEL_DIR} ${MODEL_DIR} ${FILEEXT}
test_RNAs mRNAs ${MODEL_DIR} ${MODEL_DIR} ${FILEEXT}

