#!/usr/bin/perl -w
use warnings;
use strict;
use Statistics::Descriptive;

my $nargc = $#ARGV + 1;
my @hdr = ();

if ($nargc == 1) {
	open(hdrFile, "<$ARGV[0]") or die ("Can't open the header file : $!");
	while(my $line = <hdrFile>) {
		if($line =~ /^[\%A-Za-z]/){
			chomp($line);
			@hdr = split(" ",$line);	
			close (hdrFile) or die ("can't close the header file : $!"); 
			last;
		}
	}
}

open(inFile, "<&STDIN") or die ("Can't open the input file : $!");
open(outFile, ">&STDOUT") or die ("Can't open the output file : $!");

my @pscores = ();
my @nscores = ();
my $pnum = 0;
my $nnum = 0;
my $num = 0;
my @scores = ();

while(my $line = <inFile>){
	if($line =~ /^[\-0-9]/){
		chomp($line);
		my ($label, @row) = split(" ",$line);
		($label == 1) ? $pnum++ : $nnum++;
		$num++;
		foreach my $col (@row){
			my ($left, $right) = split(":", $col);
			($label == 1) ? push(@{$pscores[$left]}, $right) : push(@{$nscores[$left]}, $right);
			push(@{$scores[$left]}, $right)
		}
	}
}
#for (my $i=0; $i < @hdr; $i++) {
#	printf (STDERR "%s\t", $hdr[$i-1]) if ($nargc == 1);
#}

my $j;
for(my $i=1; $i < @scores; $i++) {
	
	$j = 0;
	$j = scalar(@{$pscores[$i]}) if(defined $pscores[$i]);
	while($j++ < $pnum) {
		push(@{$pscores[$i]}, 0);
	}
	
	$j = 0;
	$j = scalar(@{$nscores[$i]}) if(defined $nscores[$i]);
	while($j++ < $nnum) {
		push(@{$nscores[$i]}, 0);
	}

	$j = 0;
	$j = scalar(@{$scores[$i]}) if(defined $scores[$i]);
	while($j++ < $num) {
		push(@{$scores[$i]}, 0);
	}	
}

#testParsing();
computeFscores();

sub testParsing {
	for(my $i=1; $i < @scores; $i++) {
		my $pstat = Statistics::Descriptive::Full->new();
		my $nstat = Statistics::Descriptive::Full->new();
		$pstat->add_data(@{$pscores[$i]});
		$nstat->add_data(@{$nscores[$i]});
		my ($pcount, $ncount) = ($pstat->count(), $nstat->count());
		
		print("$i"."\t");
		print(@pscores."\t".$pcount."\t");
		print("@{$pscores[$i]}");
		print("\t");
		print(@nscores."\t".$ncount."\t");
		print("@{$nscores[$i]}");
		print("\n");
	}
}

sub computeFscores {
	for(my $i=1; $i < @scores; $i++) {
		my $pstat = Statistics::Descriptive::Full->new();
		my $nstat = Statistics::Descriptive::Full->new();
		my $stat = Statistics::Descriptive::Full->new();

		$pstat->add_data(@{$pscores[$i]});
		$nstat->add_data(@{$nscores[$i]});
		$stat->add_data(@{$scores[$i]});

		my ($pcount, $ncount, $count) = ($pstat->count(), $nstat->count(), $stat->count());
		my ($pmean, $pstd, $pvar) = ($pstat->mean(), $pstat->standard_deviation(), $pstat->variance());
		my ($nmean, $nstd, $nvar) = ($nstat->mean(), $nstat->standard_deviation(), $nstat->variance());
		my ($mean, $std, $var) = ($stat->mean(), $stat->standard_deviation(), $stat->variance());
		
		printf (outFile "%s\t", $hdr[$i-1]) if ($nargc == 1 && ($i-1) < @hdr );
		printf (outFile "%u\t", $i);
		printf (outFile "%f\t", $pmean);
		printf (outFile "%f\t", $nmean);
		printf (outFile "%f\t", $mean);
		printf (outFile "%u\t", $pcount);
		printf (outFile "%u\t", $ncount);
		printf (outFile "%u\t", $count);
		printf (outFile "%f\t", $pstd);
		printf (outFile "%f\t", $nstd);
		printf (outFile "%f\t", $std);
		printf (outFile "%f\t", $pvar);
		printf (outFile "%f\t", $nvar);
		printf (outFile "%f\t", $var);

		my ($fscore1, $fscore2) = (0, 0);
		
		if (($pstd + $nstd) != 0) {
			$fscore1 = abs($pmean - $nmean)/($pstd + $nstd);
			$fscore2 = (($pmean - $mean)*($pmean - $mean) + ($nmean - $mean)*($nmean - $mean))/($pvar + $nvar);
			printf (outFile "%f\t", $fscore1);
			printf (outFile "%f", $fscore2);
		}

		printf (outFile "\n");
	}
}
close (inFile) or die ("can't close the input file : $!");
close (outFile) or die ("can't close the output file : $!"); 
